@@ -13,7 +13,6 @@ import threading
 import time
 import uuid
 from abc import ABCMeta, abstractmethod
-from io import IOBase
 from threading import Timer
 from typing import (
     IO,
@@ -40,7 +39,7 @@ from schema_salad.utils import json_dump, json_dumps
 from typing_extensions import TYPE_CHECKING
 
 from . import env_to_stdout, run_job
-from .builder import Builder, HasReqsHints
+from .builder import Builder
 from .context import RuntimeContext
 from .errors import UnsupportedRequirement, WorkflowException
 from .loghandler import _logger
@@ -50,6 +49,7 @@ from .secrets import SecretStore
 from .utils import (
     CWLObjectType,
     CWLOutputType,
+    HasReqsHints,
     DirectoryType,
     OutputCallbackType,
     bytes2str_in_dicts,
@@ -451,7 +451,6 @@ class JobBase(HasReqsHints, metaclass=ABCMeta):
         Note that with containers, the paths will (likely) be those from
         inside.
         """
-        pass
 
     def _preserve_environment_on_containers_warning(
         self, varname: Optional[Iterable[str]] = None
@@ -459,7 +458,6 @@ class JobBase(HasReqsHints, metaclass=ABCMeta):
         """When running in a container, issue a warning."""
         # By default, don't do anything; ContainerCommandLineJob below
         # will issue a warning.
-        pass
 
     def prepare_environment(
         self, runtimeContext: RuntimeContext, envVarReq: Mapping[str, str]
