@@ -1,5 +1,6 @@
 """Loads a CWL document."""
 
+import copy
 import hashlib
 import logging
 import os
@@ -307,8 +308,10 @@ def resolve_and_validate_document(
             "No cwlVersion found. "
             "Use the following syntax in your CWL document to declare "
             "the version: cwlVersion: <version>.\n"
-            "Note: if this is a CWL draft-2 (pre v1.0) document then it "
-            "will need to be upgraded first."
+            "Note: if this is a CWL draft-3 (pre v1.0) document then it "
+            "will need to be upgraded first using https://pypi.org/project/cwl-upgrader/ . "
+            "'sbg:draft-2' documents can be upgraded using "
+            "https://pypi.org/project/sevenbridges-cwl-draft2-upgrader/ ."
         )
 
     if not isinstance(cwlVersion, str):
@@ -389,6 +392,11 @@ def resolve_and_validate_document(
     if loadingContext.metadata:
         metadata = loadingContext.metadata
 
+    # Make a shallow copy.  If we do a version update later, metadata
+    # will be updated, we don't want to write through and change the
+    # original object.
+    metadata = copy.copy(metadata)
+
     if not isinstance(metadata, CommentedMap):
         raise ValidationException(
             "metadata must be a CommentedMap, was %s" % type(metadata)
