@@ -49,7 +49,7 @@ from schema_salad.validate import avro_type_name, validate_ex
 from typing_extensions import TYPE_CHECKING
 
 from . import expression
-from .builder import INPUT_OBJ_VOCAB, Builder, HasReqsHints
+from .builder import INPUT_OBJ_VOCAB, Builder
 from .context import LoadingContext, RuntimeContext, getdefault
 from .errors import UnsupportedRequirement, WorkflowException
 from .loghandler import _logger
@@ -62,6 +62,7 @@ from .utils import (
     CWLObjectType,
     CWLOutputAtomType,
     CWLOutputType,
+    HasReqsHints,
     JobsGeneratorType,
     OutputCallbackType,
     adjustDirObjs,
@@ -353,11 +354,13 @@ def relocateOutputs(
                 yield from _collectDirEntries(sub_obj)
 
     def _relocate(src: str, dst: str) -> None:
+        src = fs_access.realpath(src)
+        dst = fs_access.realpath(dst)
+
         if src == dst:
             return
 
         # If the source is not contained in source_directories we're not allowed to delete it
-        src = fs_access.realpath(src)
         src_can_deleted = any(
             os.path.commonprefix([p, src]) == p for p in source_directories
         )
