@@ -2,16 +2,16 @@ import json
 import logging
 import os
 import re
+import shutil
 import stat
 import subprocess
 import sys
-import shutil
 from io import StringIO
 from pathlib import Path
 from typing import Any, Dict, List, Union, cast
 from urllib.parse import urlparse
 
-import pydot  # type: ignore
+import pydot
 import pytest
 from ruamel.yaml.comments import CommentedMap, CommentedSeq
 from schema_salad.exceptions import ValidationException
@@ -921,6 +921,7 @@ def test_format_expr_error() -> None:
 def test_static_checker() -> None:
     # check that the static checker raises exception when a source type
     # mismatches its sink type.
+    """Confirm that static type checker raises expected exception."""
     factory = cwltool.factory.Factory()
 
     with pytest.raises(ValidationException):
@@ -933,6 +934,49 @@ def test_static_checker() -> None:
         factory.make(get_data("tests/checker_wf/broken-wf3.cwl"))
 
 
+def test_circular_dependency_checker() -> None:
+    # check that the circular dependency checker raises exception when there is
+    # circular dependency in the workflow.
+    """Confirm that circular dependency checker raises expected exception."""
+    factory = cwltool.factory.Factory()
+
+    with pytest.raises(
+        ValidationException,
+        match=r".*The\s*following\s*steps\s*have\s*circular\s*dependency:\s*.*",
+    ):
+        factory.make(get_data("tests/checker_wf/circ-dep-wf.cwl"))
+
+    with pytest.raises(
+        ValidationException,
+        match=r".*#cat-a.*",
+    ):
+        factory.make(get_data("tests/checker_wf/circ-dep-wf.cwl"))
+
+    with pytest.raises(
+        ValidationException,
+        match=r".*#ls.*",
+    ):
+        factory.make(get_data("tests/checker_wf/circ-dep-wf.cwl"))
+
+    with pytest.raises(
+        ValidationException,
+        match=r".*#wc.*",
+    ):
+        factory.make(get_data("tests/checker_wf/circ-dep-wf.cwl"))
+
+    with pytest.raises(
+        ValidationException,
+        match=r".*The\s*following\s*steps\s*have\s*circular\s*dependency:\s*.*",
+    ):
+        factory.make(get_data("tests/checker_wf/circ-dep-wf2.cwl"))
+
+    with pytest.raises(
+        ValidationException,
+        match=r".*#ls.*",
+    ):
+        factory.make(get_data("tests/checker_wf/circ-dep-wf2.cwl"))
+
+
 def test_var_spool_cwl_checker1() -> None:
     """Confirm that references to /var/spool/cwl are caught."""
     stream = StringIO()
@@ -1036,7 +1080,7 @@ def test_print_dot() -> None:
         )
     )[0]
     stdout = StringIO()
-    assert main(["--print-dot", cwl_path], stdout=stdout) == 0
+    assert main(["--debug", "--print-dot", cwl_path], stdout=stdout) == 0
     computed_dot = pydot.graph_from_dot_data(stdout.getvalue())[0]
     computed_edges = sorted(
         (urlparse(source).fragment, urlparse(target).fragment)
@@ -1066,6 +1110,7 @@ def test_js_console_cmd_line_tool(factor: str) -> None:
         )
         error_code, _, stderr = get_main_output(commands)
 
+        stderr = re.sub(r"\s\s+", " ", stderr)
         assert "[log] Log message" in stderr
         assert "[err] Error message" in stderr
 
@@ -1079,6 +1124,7 @@ def test_no_js_console(factor: str) -> None:
         commands.extend(["--no-container", get_data("tests/wf/" + test_file)])
         _, _, stderr = get_main_output(commands)
 
+        stderr = re.sub(r"\s\s+", " ", stderr)
         assert "[log] Log message" not in stderr
         assert "[err] Error message" not in stderr
 
@@ -1094,6 +1140,7 @@ def test_cid_file_dir(tmp_path: Path, factor: str) -> None:
             ["--cidfile-dir", str(tmp_path), get_data("tests/wf/" + test_file)]
         )
         error_code, stdout, stderr = get_main_output(commands)
+        stderr = re.sub(r"\s\s+", " ", stderr)
         assert "completed success" in stderr
         assert error_code == 0
         cidfiles_count = sum(1 for _ in tmp_path.glob("**/*"))
@@ -1112,6 +1159,7 @@ def test_cid_file_dir_arg_is_file_instead_of_dir(tmp_path: Path, factor: str) ->
         ["--cidfile-dir", str(bad_cidfile_dir), get_data("tests/wf/" + test_file)]
     )
     error_code, _, stderr = get_main_output(commands)
+    stderr = re.sub(r"\s\s+", " ", stderr)
     assert "is not a directory, please check it first" in stderr, stderr
     assert error_code == 2 or error_code == 1, stderr
 
@@ -1132,6 +1180,7 @@ def test_cid_file_non_existing_dir(tmp_path: Path, factor: str) -> None:
         ]
     )
     error_code, _, stderr = get_main_output(commands)
+    stderr = re.sub(r"\s\s+", " ", stderr)
     assert "directory doesn't exist, please create it first" in stderr, stderr
     assert error_code == 2 or error_code == 1, stderr
 
@@ -1155,6 +1204,7 @@ def test_cid_file_w_prefix(tmp_path: Path, factor: str) -> None:
         finally:
             listing = tmp_path.iterdir()
             cidfiles_count = sum(1 for _ in tmp_path.glob("**/pytestcid*"))
+    stderr = re.sub(r"\s\s+", " ", stderr)
     assert "completed success" in stderr
     assert error_code == 0
     assert cidfiles_count == 2, f"{list(listing)}/n{stderr}"
@@ -1180,6 +1230,7 @@ def test_secondary_files_v1_1(factor: str) -> None:
         # 664 in octal, '-rw-rw-r--'
         assert stat.S_IMODE(os.stat("lsout").st_mode) == 436
         os.umask(old_umask)  # revert back to original umask
+    stderr = re.sub(r"\s\s+", " ", stderr)
     assert "completed success" in stderr
     assert error_code == 0
 
@@ -1222,6 +1273,7 @@ def test_secondary_files_v1_0(tmp_path: Path, factor: str) -> None:
         ]
     )
     error_code, _, stderr = get_main_output(commands)
+    stderr = re.sub(r"\s\s+", " ", stderr)
     assert "completed success" in stderr
     assert error_code == 0
 
@@ -1246,6 +1298,7 @@ def test_wf_without_container(tmp_path: Path, factor: str) -> None:
     )
     error_code, _, stderr = get_main_output(commands)
 
+    stderr = re.sub(r"\s\s+", " ", stderr)
     assert "completed success" in stderr
     assert error_code == 0
 
@@ -1260,6 +1313,7 @@ def test_issue_740_fixed(tmp_path: Path, factor: str) -> None:
     commands.extend(["--cachedir", cache_dir, get_data("tests/wf/" + test_file)])
     error_code, _, stderr = get_main_output(commands)
 
+    stderr = re.sub(r"\s\s+", " ", stderr)
     assert "completed success" in stderr
     assert error_code == 0
 
@@ -1267,6 +1321,7 @@ def test_issue_740_fixed(tmp_path: Path, factor: str) -> None:
     commands.extend(["--cachedir", cache_dir, get_data("tests/wf/" + test_file)])
     error_code, _, stderr = get_main_output(commands)
 
+    stderr = re.sub(r"\s\s+", " ", stderr)
     assert "Output of job will be cached in" not in stderr
     assert error_code == 0, stderr
 
@@ -1355,6 +1410,7 @@ def test_no_compute_chcksum(tmp_path: Path, factor: str) -> None:
         ]
     )
     error_code, stdout, stderr = get_main_output(commands)
+    stderr = re.sub(r"\s\s+", " ", stderr)
     assert "completed success" in stderr
     assert error_code == 0
     assert "checksum" not in stdout
@@ -1374,6 +1430,7 @@ def test_bad_userspace_runtime(factor: str) -> None:
         ]
     )
     error_code, stdout, stderr = get_main_output(commands)
+    stderr = re.sub(r"\s\s+", " ", stderr)
     assert "or quaquioN is missing or broken" in stderr, stderr
     assert error_code == 1
 
@@ -1384,6 +1441,7 @@ def test_bad_basecommand(factor: str) -> None:
     commands = factor.split()
     commands.extend([get_data(test_file)])
     error_code, stdout, stderr = get_main_output(commands)
+    stderr = re.sub(r"\s\s+", " ", stderr)
     assert "'neenooGo' not found" in stderr, stderr
     assert error_code == 1
 
@@ -1406,6 +1464,7 @@ def test_v1_0_position_expression(factor: str) -> None:
     commands = factor.split()
     commands.extend(["--debug", get_data(test_file), get_data(test_job)])
     error_code, stdout, stderr = get_main_output(commands)
+    stderr = re.sub(r"\s\s+", " ", stderr)
     assert "is not int" in stderr, stderr
     assert error_code == 1
 
@@ -1430,6 +1489,7 @@ def test_optional_numeric_output_0(factor: str) -> None:
     commands.extend([get_data(test_file)])
     error_code, stdout, stderr = get_main_output(commands)
 
+    stderr = re.sub(r"\s\s+", " ", stderr)
     assert "completed success" in stderr
     assert error_code == 0
     assert json.loads(stdout)["out"] == 0
@@ -1463,6 +1523,7 @@ def test_env_filtering(factor: str) -> None:
     assert sh_name_b
     sh_name = sh_name_b.decode("utf-8").strip()
 
+    stderr = re.sub(r"\s\s+", " ", stderr)
     assert "completed success" in stderr, (error_code, stdout, stderr)
     assert error_code == 0, (error_code, stdout, stderr)
     if sh_name == "dash":
@@ -1484,6 +1545,7 @@ def test_v1_0_arg_empty_prefix_separate_false() -> None:
     error_code, stdout, stderr = get_main_output(
         ["--debug", get_data(test_file), "--echo"]
     )
+    stderr = re.sub(r"\s\s+", " ", stderr)
     assert "completed success" in stderr
     assert error_code == 0
 
@@ -1515,7 +1577,7 @@ def test_malformed_hints() -> None:
     factory = cwltool.factory.Factory()
     with pytest.raises(
         ValidationException,
-        match=r".*wc-tool-bad-hints\.cwl:6:1: If 'hints' is\s*present\s*then\s*it\s*must\s*be\s*a\s*list.*",
+        match=r".*wc-tool-bad-hints\.cwl:6:1:\s*If\s*'hints'\s*is\s*present\s*then\s*it\s*must\s*be\s*a\s*list.*",
     ):
         factory.make(get_data("tests/wc-tool-bad-hints.cwl"))
 
@@ -1525,7 +1587,7 @@ def test_malformed_reqs() -> None:
     factory = cwltool.factory.Factory()
     with pytest.raises(
         ValidationException,
-        match=r".*wc-tool-bad-reqs\.cwl:6:1: If 'requirements' is\s*present\s*then\s*it\s*must\s*be\s*a\s*list.*",
+        match=r".*wc-tool-bad-reqs\.cwl:6:1:\s*If\s*'requirements'\s*is\s*present\s*then\s*it\s*must\s*be\s*a\s*list.*",
     ):
         factory.make(get_data("tests/wc-tool-bad-reqs.cwl"))
 
